@@ -4,7 +4,7 @@
 
 ;; Author: Hugo Thunnissen <devel@hugot.nl>
 ;; Keywords: php, languages, tools, convenience
-;; Version: 1.1.0
+;; Version: 1.2.0
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -134,17 +134,24 @@ serious performance hits. Enable at your own risk (:")
     (puthash (phpinspect--function-name-symbol func) func
              (phpinspect-project-function-index project))))
 
+(cl-defmethod phpinspect-project-get-function
+  ((project phpinspect-project) (name string))
+  (phpinspect-project-get-function project (phpinspect-intern-name name)))
+
 (cl-defmethod phpinspect-project-get-function
   ((project phpinspect-project) (name (head phpinspect-name)))
   (gethash name (phpinspect-project-function-index project)))
 
-(cl-defmethod phpinspect-project-get-function-or-extra
-  ((project phpinspect-project) (name (head phpinspect-name)))
+(cl-defmethod phpinspect-project-get-function-or-extra ((project phpinspect-project) name)
   (or (phpinspect-project-get-function project name)
       (and (phpinspect-project-extra-function-retriever project)
            (funcall (phpinspect-project-extra-function-retriever project)
                     name))))
 
+(defun phpinspect-project-get-function-return-type (project function-name)
+  (when-let ((fn (phpinspect-project-get-function-or-extra project function-name)))
+    (phpinspect--function-return-type fn)))
+
 (cl-defmethod phpinspect-project-delete-function
   ((project phpinspect-project) (name (head phpinspect-name)))
   (phpinspect-project-edit project
